@@ -26,6 +26,7 @@
 #include <gconf/gconf-client.h>
 #include <games-frame.h>
 #include <games-controls.h>
+#include <games-files.h>
 
 #include "properties.h"
 #include "gameconfig.h"
@@ -54,6 +55,8 @@
 #define KEY_ENABLE_SOUND     "/apps/gnobots2/preferences/enable_sound"
 #define KEY_ENABLE_SPLATS    "/apps/gnobots2/preferences/enable_splats"
 #define KEY_BACKGROUND_COLOR "/apps/gnobots2/preferences/background_color"
+#define KEY_WINDOW_WIDTH     "/apps/gnobots2/geometry/width"
+#define KEY_WINDOW_HEIGHT    "/apps/gnobots2/geometry/height"
 
 /**********************************************************************/
 
@@ -70,9 +73,9 @@ struct _GnobotsProperties {
   gboolean splats;
   gboolean show_toolbar;
   GdkColor bgcolour;
-  gint     selected_graphics;
   gint     selected_config;
   gint     keys[12];
+  gchar    *themename;
 };
 /**********************************************************************/
 
@@ -81,13 +84,8 @@ struct _GnobotsProperties {
 /* File Static Variables                                              */
 /**********************************************************************/
 static GtkWidget         *propbox      = NULL;
-static GtkWidget         *list_view    = NULL;
-enum {
-  PROPERTY_STRING,
-  VALUE_STRING,
-  INDEX_INT,
-  NCOLS
-}; /* Column indices for list */
+
+static GamesFileList     *theme_list   = NULL;
 
 static GnobotsProperties  properties;
 
@@ -109,7 +107,6 @@ static GConfClient *gconf_client;
 static void apply_changes (void);
 static void apply_cb (GtkWidget*, gpointer);
 static gboolean delete_cb (GtkWidget*, gpointer);
-static void fill_property_list (void);
 static void pmap_selection (GtkWidget*, gpointer);
 static void type_selection (GtkWidget*, gpointer);
 static void safe_cb (GtkWidget*, gpointer);
@@ -117,7 +114,6 @@ static void sound_cb (GtkWidget*, gpointer);
 static void splat_cb (GtkWidget*, gpointer);
 static void defkey_cb (GtkWidget*, gpointer);
 static void fill_typemenu (GtkWidget*);
-static void fill_pmapmenu (GtkWidget*);
 static void gconf_set_background_color (GdkColor * c);
 /**********************************************************************/
 
@@ -156,7 +152,6 @@ apply_cb (GtkWidget *w, gpointer data)
   apply_changes ();
 
   gtk_widget_destroy (propbox);
-  list_view = NULL;
   propbox = NULL;
 }
 
@@ -172,294 +167,28 @@ apply_cb (GtkWidget *w, gpointer data)
 static gboolean
 delete_cb (GtkWidget *w, gpointer data)
 {
-  list_view = NULL;
   propbox = NULL;
 
   return FALSE;
 }
 
-
-/**
- * fill_property_list
- *
- * Description:
- * Fills the property list of configuration values
- **/
-static void
-fill_property_list (void)
+gint save_window_geometry (GtkWidget *w, GdkEventConfigure *e, gpointer data)
 {
-  gchar *entry[2];
-  gchar buffer[256];
-  GameConfig *gc;
-  gboolean type2exist = TRUE;
-  GtkListStore *list;
-  GtkTreeIter iter;
-  int index = 0;
-
-  if (list_view == NULL) return;
-
-  list = (GtkListStore *) gtk_tree_view_get_model (GTK_TREE_VIEW (list_view));
-
-  gc = game_config_settings (properties.selected_config);
-
-  if ((gc->initial_type2 <= 0) && 
-      (gc->increment_type2 <= 0) && 
-      (gc->maximum_type2 <= 0)){
-    type2exist = FALSE;
-  }
-
-  gtk_list_store_clear (list);
-  /* I think this is just gtk_list_store_clear () */
-  
-  if (type2exist){
-    entry[0] = _("Initial Number of Type 1 Robots");
-  } else {
-    entry[0] = _("Initial Number of Robots");
-  }
-  sprintf (buffer, "%d", gc->initial_type1);
-  entry[1] = buffer;
-
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  if (type2exist){
-    entry[0] = _("Initial Number of Type 2 Robots");
-    sprintf (buffer, "%d", gc->initial_type2);
-    entry[1] = buffer;
-    gtk_list_store_append (list, &iter);
-    gtk_list_store_set (list, &iter,
-                        PROPERTY_STRING, entry[0],
-                        VALUE_STRING, entry[1],
-                        INDEX_INT, index,
-                        -1);
-    index++;
-  }
-
-  if (type2exist){
-    entry[0] = _("Increasing Number of Type 1 Robots per Level");
-  } else {
-    entry[0] = _("Increasing Number of Robots per Level");
-  }
-  sprintf (buffer, "%d", gc->increment_type1);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  if (type2exist){
-    entry[0] = _("Increasing Number of Type 2 Robots per Level");
-    sprintf (buffer, "%d", gc->increment_type2);
-    entry[1] = buffer;
-    gtk_list_store_append (list, &iter);
-    gtk_list_store_set (list, &iter,
-                        PROPERTY_STRING, entry[0],
-                        VALUE_STRING, entry[1],
-                        INDEX_INT, index,
-                        -1);
-    index++;
-  }
-
-  if (type2exist){
-    entry[0] = _("Maximum Number of Type 1 Robots");
-  } else {
-    entry[0] = _("Maximum Number of Robots");
-  }
-  if (gc->maximum_type1 > MAX_ROBOTS){
-    sprintf (buffer, _("Fill Screen"));
-  } else {
-    sprintf (buffer, "%d", gc->maximum_type1);
-  }
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  if (type2exist){
-    entry[0] = _("Maximum Number of Type 2 Robots");
-    if (gc->maximum_type2 > MAX_ROBOTS){
-      sprintf (buffer, _("Fill Screen"));
-    } else {
-      sprintf (buffer, "%d", gc->maximum_type2);
-    }
-    entry[1] = buffer;
-    gtk_list_store_append (list, &iter);
-    gtk_list_store_set (list, &iter,
-                        PROPERTY_STRING, entry[0],
-                        VALUE_STRING, entry[1],
-                        INDEX_INT, index,
-                        -1);
-    index++;
-  }
-
-  if (type2exist){
-    entry[0] = _("Type 1 Robot Score");
-  } else {
-    entry[0] = _("Robot Score");
-  }
-  sprintf (buffer, "%d", gc->score_type1);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  if (type2exist){
-    entry[0] = _("Type 2 Robot Score");
-    sprintf (buffer, "%d", gc->score_type2);
-    entry[1] = buffer;
-    gtk_list_store_append (list, &iter);
-    gtk_list_store_set (list, &iter,
-                        PROPERTY_STRING, entry[0],
-                        VALUE_STRING, entry[1],
-                        INDEX_INT, index,
-                        -1);
-    index++;
-  }
-
-  if (type2exist){
-    entry[0] = _("Type 1 Robot Score When Waiting");
-  } else {
-    entry[0] = _("Robot Score When Waiting");
-  }
-  sprintf (buffer, "%d", gc->score_type1_waiting);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  if (type2exist){
-    entry[0] = _("Type 2 Robot Score When Waiting");
-    sprintf (buffer, "%d", gc->score_type2_waiting);
-    entry[1] = buffer;
-    gtk_list_store_append (list, &iter);
-    gtk_list_store_set (list, &iter,
-                        PROPERTY_STRING, entry[0],
-                        VALUE_STRING, entry[1],
-                        INDEX_INT, index,
-                        -1);
-    index++;
-  }
-
-  if (type2exist){
-    entry[0] = _("Type 1 Robot Score When Splatted");
-  } else {
-    entry[0] = _("Robot Score When Splatted");
-  }
-  sprintf (buffer, "%d", gc->score_type1_splatted);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  if (type2exist){
-    entry[0] = _("Type 2 Robot Score When Splatted");
-    sprintf (buffer, "%d", gc->score_type2_splatted);
-    entry[1] = buffer;
-    gtk_list_store_append (list, &iter);
-    gtk_list_store_set (list, &iter,
-                        PROPERTY_STRING, entry[0],
-                        VALUE_STRING, entry[1],
-                        INDEX_INT, index,
-                        -1);
-    index++;
-  }
-
-  entry[0] = _("Initial Number of Safe Teleports");
-  sprintf (buffer, "%d", gc->initial_safe_teleports);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  entry[0] = _("Number of Free Safe Teleports per Level");
-  sprintf (buffer, "%d", gc->free_safe_teleports);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  entry[0] = _("Kills Required While Waiting to Get A Safe Teleport");
-  sprintf (buffer, "%d", gc->num_robots_per_safe);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  entry[0] = _("Score Required To Get A Safe Teleport");
-  sprintf (buffer, "%d", gc->safe_score_boundary);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  entry[0] = _("Maximum Number of Safe Teleports");
-  sprintf (buffer, "%d", gc->max_safe_teleports);
-  entry[1] = buffer;
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-  index++;
-
-  entry[0] = _("Moveable Junkheaps");
-  if (gc->moveable_heaps) {
-    entry[1] = _("Yes");
-  } else {
-    entry[1] = _("No");
-  }
-  gtk_list_store_append (list, &iter);
-  gtk_list_store_set (list, &iter,
-                      PROPERTY_STRING, entry[0],
-                      VALUE_STRING, entry[1],
-                      INDEX_INT, index,
-                      -1);
-
-  /* gtk_tree_view_set_model (list_view, list); */
+  gconf_client_set_int (gconf_client, KEY_WINDOW_WIDTH, e->width, NULL);
+  gconf_client_set_int (gconf_client, KEY_WINDOW_HEIGHT, e->height, NULL);
 
+  return FALSE;
 }
 
+void set_window_geometry (GtkWidget *window)
+{
+  int w, h;
+
+  w = gconf_client_get_int (gconf_client, KEY_WINDOW_WIDTH, NULL);
+  h = gconf_client_get_int (gconf_client, KEY_WINDOW_HEIGHT, NULL);
+
+  gtk_window_resize (GTK_WINDOW (window), w, h);
+}
 
 /**
  * pmap_selection
@@ -472,13 +201,16 @@ fill_property_list (void)
 static void
 pmap_selection (GtkWidget *widget, gpointer data)
 {
-  gint num = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+  gint n;
 
-  properties.selected_graphics = num;
+  n = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 
-  gconf_set_theme (game_graphics_name (properties.selected_graphics));
+  /* FIXME: Should be de-suffixed. */
+  properties.themename = games_file_list_get_nth (theme_list, n);
 
-  set_game_graphics (properties.selected_graphics);
+  gconf_set_theme (properties.themename);
+
+  set_game_graphics (properties.themename);
   clear_game_area ();
 }
 
@@ -497,7 +229,6 @@ type_selection (GtkWidget *widget, gpointer data)
   gint num = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 
   properties.selected_config = num;
-  fill_property_list ();
 
   gconf_set_configuration (game_config_name (properties.selected_config));
 
@@ -633,10 +364,17 @@ fill_typemenu (GtkWidget *menu)
  * Description:
  * fills the listbox with pixmap names
  **/
-static void
-fill_pmapmenu (GtkWidget *menu)
+static GtkWidget *
+make_theme_menu (void)
 {
-  gint i;
+  if (theme_list)
+    g_object_unref (theme_list);
+
+  theme_list = games_file_list_new_images (PIXMAPDIR, NULL);
+  games_file_list_transform_basename (theme_list);
+
+  /* FIXME: Get rid of the bubbles images from the list (preferably by
+   * getting tid of the bubble pixmaps. */
 
 #if 0
   /* this is just a place holder so that xgettext can found the strings to
@@ -648,18 +386,15 @@ fill_pmapmenu (GtkWidget *menu)
     N_("eggs"),
     N_("gnomes"),
     N_("mice"),
-    N_("windows"),
+    N_("ufo"),
+    N_("boo"),
   };
 #endif
 
-  for (i = 0; i < num_game_graphics (); ++i) {
-    gtk_combo_box_append_text (GTK_COMBO_BOX (menu),
-                               _(game_graphics_name (i)));
-  }
-
-  gtk_combo_box_set_active (GTK_COMBO_BOX (menu),
-                            properties.selected_graphics);
-
+  return games_file_list_create_widget (theme_list, 
+					properties.themename,
+					GAMES_FILE_LIST_REMOVE_EXTENSION |
+					GAMES_FILE_LIST_REPLACE_UNDERSCORES);
 }
 
 static void
@@ -672,6 +407,12 @@ bg_color_callback (GtkWidget *widget, gpointer data)
   gconf_set_background_color (&properties.bgcolour);
 }
 
+gchar *
+properties_theme_name (void)
+{
+  return properties.themename;
+}
+
 /**
  * show_properties_dialog
  *
@@ -693,15 +434,10 @@ show_properties_dialog (void)
   GtkWidget *chkbox;
   GtkWidget *table;
   GtkWidget *dbut;
-  GtkWidget *scrolled;
   GtkWidget *frame;
   GtkWidget *w;
   GtkWidget *controls_list;
   GtkTooltips *tooltips;
-  GtkListStore *list;
-  GtkTreeViewColumn *column;
-  GtkCellRenderer *renderer;
-  GtkTreeSelection *selection;
 
   if (propbox) 
     return;
@@ -744,43 +480,6 @@ show_properties_dialog (void)
   fill_typemenu (typemenu);
   gtk_box_pack_start_defaults (GTK_BOX (hbox), typemenu);
   
-  list = gtk_list_store_new (NCOLS,
-			     G_TYPE_STRING, /* Property */ 
-			     G_TYPE_STRING, /* Value */
-			     G_TYPE_INT); /* Index - remains hidden */
-  /* Create view */
-  list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list));
-  
-  renderer = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes (_("Property"), 
-						     renderer, 
-						     "text", PROPERTY_STRING, 
-						     NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), column);
-    
-  renderer = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes (_("Value"), 
-						     renderer, 
-						     "text", VALUE_STRING, 
-						     NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), column);
-
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
-  gtk_tree_selection_set_mode  (selection, GTK_SELECTION_SINGLE);
-
-  fill_property_list ();
-
-  scrolled = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-				  GTK_POLICY_AUTOMATIC,
-				  GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type 
-    (GTK_SCROLLED_WINDOW (scrolled), GTK_SHADOW_IN);
-  gtk_widget_set_size_request (scrolled, 400, 200);
-  gtk_container_add (GTK_CONTAINER (scrolled), list_view);
-
-  gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
-
   frame = games_frame_new (_("Options"));
   gtk_box_pack_start (GTK_BOX (cpage), frame, FALSE, FALSE, 0);
   vbox = gtk_vbox_new (TRUE, 6);
@@ -848,12 +547,11 @@ show_properties_dialog (void)
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
 
-  pmapmenu = gtk_combo_box_new_text ();
+  pmapmenu = make_theme_menu ();
   g_signal_connect (G_OBJECT (pmapmenu), "changed",
                     G_CALLBACK (pmap_selection), NULL);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), pmapmenu);
 
-  fill_pmapmenu (pmapmenu);
   gtk_table_attach_defaults (GTK_TABLE (table), pmapmenu, 1, 2, 0, 1);
 
   label = gtk_label_new_with_mnemonic (_("_Background color:"));
@@ -950,7 +648,6 @@ gboolean
 load_properties (void)
 {
   gchar buffer[256];
-  gchar *sname = NULL;
   gchar *cname = NULL;
   gint i;
   gchar *str;
@@ -979,18 +676,10 @@ load_properties (void)
   gdk_color_parse (bgcolour, &properties.bgcolour);
   set_background_color (properties.bgcolour);
   
-  sname = gconf_client_get_string (get_gconf_client (), KEY_THEME, NULL);
-  if (sname == NULL)
-    sname = g_strdup ("robots");
-
-  properties.selected_graphics = 0;
-  for (i = 0; i < num_game_graphics (); ++i) {
-    if (! strcmp (sname, game_graphics_name (i))) {
-      properties.selected_graphics = i;
-      break;
-    }
-  }
-  g_free (sname);
+  properties.themename = gconf_client_get_string (get_gconf_client (), 
+						  KEY_THEME, NULL);
+  if (properties.themename == NULL)
+    properties.themename = ("robots");
 
   cname = gconf_client_get_string (get_gconf_client (), KEY_CONFIGURATION, NULL);
   if (cname == NULL)
@@ -1016,7 +705,7 @@ load_properties (void)
   properties.show_toolbar = gconf_client_get_bool (get_gconf_client (),
 						   KEY_SHOW_TOOLBAR, NULL);
 
-  set_game_graphics (properties.selected_graphics);
+  set_game_graphics (properties.themename);
   set_game_config (properties.selected_config);
   keyboard_set (properties.keys);
   update_score_state ();
@@ -1113,7 +802,7 @@ save_properties (void)
     gconf_set_control_key (i, gdk_keyval_name (properties.keys[i]));
   }
   
-  gconf_set_theme (game_graphics_name (properties.selected_graphics));
+  gconf_set_theme (properties.themename);
   gconf_set_configuration (game_config_name (properties.selected_config));
   gconf_set_use_safe_moves (properties.safe_moves);
   gconf_set_use_super_safe_moves (properties.super_safe_moves);
